
# ----------------------------------------
# CUTTING STOCK USING PATTERNS
# ----------------------------------------

param roll_width > 0;                       # width of raw rolls (W)
 
set WIDTHS;                                 # set of widths to be cut (L_i)
param orders {WIDTHS} > 0;                  # number of each width to be cut (R_i)

param nPAT integer >= 0;                    # number of patterns    (dynamically changed...)
set PATTERNS := 1..nPAT;                    # set of patterns

param nbr {WIDTHS,PATTERNS} integer >= 0;   # N_ij                  (dynamically generated...)

   check {j in PATTERNS}: 
      sum {i in WIDTHS} i * nbr[i,j] <= roll_width;     # remark! i = L_i !!!

                            # defn of patterns: nbr[i,j] = number
                            # of rolls of width i in pattern j

var Cut {PATTERNS} integer >= 0;   # rolls cut using each pattern (x_j)

minimize Number:                   # minimize total raw rolls cut
   sum {j in PATTERNS} Cut[j];   

subj to Fill {i in WIDTHS}:
   sum {j in PATTERNS} nbr[i,j] * Cut[j] >= orders[i];

                                   # for each width, total
                                   # rolls cut meets total orders

# ----------------------------------------
# KNAPSACK SUBPROBLEM FOR CUTTING STOCK
# ----------------------------------------

param price {WIDTHS} default 0.0;       # dual variables (u_i)

var Use {WIDTHS} integer >= 0;          # knapsack variable (z_i)

minimize Reduced_Cost:  
   1 - sum {i in WIDTHS} price[i] * Use[i];

subj to Width_Limit:  
   sum {i in WIDTHS} i * Use[i] <= roll_width;
